#!/bin/bash

renderer=fdp
image_format=png
intput_file=

basedir=`dirname "$0"`
this=`basename "$0"`

usage () {
   cat <<EOF
Usage:
  $this [options] input_file [output_file]

  Options:
    --format=image_format - set the image format (png, jpg, gif, pdf, ps, ...)
    --version | -v        - show graphviz version
    --help | -h           - show this message
EOF
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --version | -v )
      $renderer -v
      echo ""
      exit 0
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    --format | --format= )
      echo "--format option should be followed by the image format"
      echo ""
      usage
      exit 1
    ;;
    --format=* )
      image_format=`echo $1 | sed "s/--format=//"`
    ;;
    * )
      if [ "x" == "x$input_file" ]
      then
          input_file="$1"
      else
          output_file="$1"
      fi
    ;;
  esac
  shift
done

if [ "x" == "x$output_file" ]
then
    output_file=$input_file.$image_format
fi

if [ "x" == "x$input_file" ]
then
    echo "Input file is not specified"
    echo ""
    usage
    exit 1
fi

$renderer $input_file -T$image_format -o $output_file

