#!/bin/bash

basedir=`dirname "$0"`
this=`basename "$0"`

usage () {
   cat <<EOF
Usage:
  $this [options] trace_file.utt2 [another_trace_file.utt2 yet_another_trace_file.utt2 ...]

  Report generator processes UTT2-compatible traces into HTML representation.

  Options:
    --dest=output_directory   - set the output directory (default value is Reports/Report_\$(current_time))
    --disable-aspect-error    - do not process <error> aspect
    --disable-aspect-fsm      - do not process <fsm> aspect
    --version | -v            - show version number of the report generator
    --help | -h               - show this message
EOF
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --version | -v )
      echo $this 0.1.1
      echo ""
      echo "Copyright 2011 (c) ISPRAS"
      echo "Institute for System Programming of the Russian Academy of Sciences"
      echo "25 A.Solzhenitsyn st. Moscow 109004 Russia"
      echo "http://hardware.ispras.ru"
      echo ""
      exit 0
    ;;
    --help | -h )
      usage
      exit 0
    ;;

    --dest | --dest= )
      echo "--dest option should be followed by a directory name"
      echo ""
      usage
      exit 1
    ;;
    --dest=* )
      value=`echo $1 | sed "s/--dest=//"`
      reportgen_options="$reportgen_options -d $value"
      #mkdir -p $value
    ;;

    --disable-aspect-error )
      reportgen_options="$reportgen_options -e"
    ;;
    --disable-aspect-fsm )
      reportgen_options="$reportgen_options -f"
    ;;

    *.utt2 )
      if [ "x" == "x$reportgen_traces" ]
      then
        reportgen_traces="$1"
      else
        reportgen_traces="$reportgen_traces $1"
      fi
    ;;

    * )
      usage
      exit 0
  esac
  shift
done

if [ "x" == "x$reportgen_traces" ]
then
    usage
    exit 0
fi

for file in $reportgen_traces
do
  if [ -f $file ]
  then
    PROCESS_IS_GOING_ON=1
  else
    tput setaf 1
    echo File $file is missing!
    tput sgr0
    SOME_FILES_ARE_MISSING="yes"
  fi
done

if [ "x" != "x$SOME_FILES_ARE_MISSING" ]
then
    exit 1
fi

java -jar $basedir/jars/run.jar $reportgen_options $reportgen_traces

