`timescale 1ns/100ps

module arbiter(
DI0,
DI1,
DI_VAL0,
DI_VAL1,
IS_EMPTY0,
IS_EMPTY1,
IS_EMPTY2,
IS_FULL0,
IS_FULL1,
IS_FULL2,
CLK,
RST,
RQST,
DO,
CE_RD0,
CE_RD1,
CE_WR
);

parameter WIDTH = 64;

parameter FSM_START_STATE               = 0;
parameter REQUEST_WAS_RECEIVED          = 1;
parameter REQUESTS_WERE_SENT_TO_INPUTS  = 2;
parameter REQUEST_IS_WAITING_FOR_ANSWER = 3;
parameter REQUEST_WAS_STORAGED          = 4;


input [WIDTH-1:0] DI0; // input data from channel 0
input [WIDTH-1:0] DI1; // input data from channel 1
input DI_VAL0;         // input request to receive data from channel 0
input DI_VAL1;         // input request to receive data from channel 1
input IS_EMPTY0;       // info from input buffer0, is the buffer empty
input IS_EMPTY1;       // info from input buffer1, is the buffer empty
input IS_EMPTY2;       // info from output buffer, is the buffer empty
input IS_FULL0;        // info from input buffer0, is the buffer full
input IS_FULL1;        // info from input buffer1, is the buffer full
input IS_FULL2;        // info from output buffer, is the buffer full
input CLK;             // cycle signal
input RST;             // reset signal
input RQST;             // request to read data from output fifo
output [WIDTH-1:0] DO; // output data to single channel
output CE_RD0;         // chip enable - read from channel 0
output CE_RD1;         // chip enable - read from channel 1
output CE_WR;          // chip enable - write to output single channel

wire [WIDTH-1:0] DO;   // output data to single channel
wire CE_RD0;           // chip enable - read from channel 0
wire CE_RD1;           // chip enable - read from channel 1
wire CE_WR;            // chip enable - write to output single channel

reg [WIDTH-1:0] storage;
reg [2:0] state_machine;

reg request0;
reg request1;

always @(posedge CLK)
begin
    if(RST) begin
        storage = 0;
        state_machine = FSM_START_STATE;
        request0 = 0;
        request1 = 0;
    end
end

// work cycle
always @(posedge CLK)
begin
    if(state_machine == REQUEST_WAS_STORAGED)
    begin
        storage = 0;
        state_machine = FSM_START_STATE;
    end

/*    if(RQST & (state_machine == FSM_START_STATE || state_machine == REQUEST_WAS_STORAGED))
    begin
        state_machine = REQUEST_WAS_RECEIVED;
        if(!IS_EMPTY0 || !IS_EMPTY1)
        begin
            state_machine = REQUESTS_WERE_SENT_TO_INPUTS;
        end
    end

    if(state_machine == REQUESTS_WERE_SENT_TO_INPUTS)
    begin
        state_machine = REQUEST_IS_WAITING_FOR_ANSWER;
    end

    if(state_machine == REQUEST_IS_WAITING_FOR_ANSWER && (DI_VAL0 || DI_VAL1))
    begin
        state_machine = FSM_START_STATE;
    end*/

    if(DI_VAL0 && DI_VAL1)
    begin
        storage = DI1;
        state_machine = REQUEST_WAS_STORAGED;
    end

    request0 = ~IS_FULL2;
    request1 = ~IS_FULL2 && ~DI_VAL0;
end

assign DO = DI_VAL0 ? DI0 : DI_VAL1 ? DI1 : state_machine == REQUEST_WAS_STORAGED ? storage : 0;
//assign CE_RD0 = state_machine == REQUESTS_WERE_SENT_TO_INPUTS;
//assign CE_RD1 = state_machine == REQUESTS_WERE_SENT_TO_INPUTS;
assign CE_WR = DI_VAL0 ? 1 : DI_VAL1 ? 1 : state_machine == REQUEST_WAS_STORAGED ? 1 : 0;

assign CE_RD0 = ~IS_FULL2; //request0
assign CE_RD1 = ~IS_FULL2;// && ~DI_VAL0; //request1

endmodule
