/** @file fifo_model.h
 * United model of FIFOs.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/model.hpp>

#include <fifo_msg.h>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief Reference model of FIFO.
 */
CPPTESK_MODEL(FIFO) {
public:
    FIFO();
    virtual ~FIFO();

    /// Input interface iface1 to send stimuli "write".
    CPPTESK_DECLARE_INPUT(iface1, InputData);
    /// Input interface iface2 to send stimuli "read".
    CPPTESK_DECLARE_INPUT(iface2, InputData);
    /// Output interface iface3 to read results of reading.
    CPPTESK_DECLARE_OUTPUT(iface3, OutputData);

    /// Check whether iface1 is free. @return iface1.isReady()
    virtual bool isIface1Ready() const;
    /// Check whether iface2 is free. @return iface2.isReady()
    virtual bool isIface2Ready() const;

    /// Stimulus "Put data into the buffer".
    CPPTESK_DECLARE_STIMULUS(push_msg);
    /// Stimulus "Start reading from the buffer".
    CPPTESK_DECLARE_STIMULUS(pop_msg);
    /// Reaction "Read data sending by the buffer".
    CPPTESK_DECLARE_REACTION(get_pop_msg);

    /// Push data into reference FIFO model.
    void push_item(uint64_t data);
    /// Pop and remove data from reference FIFO model @return Data according to FIFO strategy.
    uint64_t pop_item(void);
    /// Check whether reference FIFO is empty @return size() == 0
    bool is_empty(void);
    /// Check whether reference FIFO is full @return size() == 16
    bool is_full(void);
    /// Find out the count of filled items in reference model of FIFO @return size()
    int fifo_fulness(void);

protected:
    /// Model of FIFO.
    std::vector<uint64_t> fifo;
};

}}}

