/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru), Mikhail Chupilko (chupilko@ispras.ru)
// SYNOPSIS: Reaction arbiters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/arbiter/base.hpp>

namespace cpptesk {
namespace hw {

////////////////////////////////////////////////////////////////////////////////
// FIFO Reaction Arbiter
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_FIFO_ARBITER ::cpptesk::hw::FifoArbiter

#ifdef HW_SHORT_NAMES
    #define FIFO_ARBITER CPPTESK_FIFO_ARBITER
#endif // HW_SHORT_NAMES

class FifoArbiter: public ModelBasedArbiter
{
public:
    FifoArbiter() {}
    virtual ~FifoArbiter() {}

    virtual void appendRequest(const Process &process, const Message &message)
    {
        _requests.push_back(ArbiterRequest(process, message));
    }

    virtual bool chooseRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(message);
        CPPTESK_ASSERTION(!_requests.empty(), "There are no requests to choose from");

        return _requests.begin()->process == &process;
    }

    virtual void removeRequest(const Process &process, const Message &message)
    {
        CPPTESK_UNUSED_ARG(message);
        CPPTESK_ASSERTION(!_requests.empty(), "There are no requests to remove");

        for(requests_t::iterator i = _requests.begin(); i != _requests.end(); i++)
        {
            if(i->process == &process)
            {
                _requests.erase(i);
                break;
            }
        }
    }
};

}} // namespace cpptesk::hw

