/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Metric for rules
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <string>

#include <hw/diagnostics/rules/rule.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

template <typename T1>
class SingleMetric
{
protected:
    /// Constructor for Metric singletone
    SingleMetric() {};
    /// Destructor for Metric singletone
    virtual ~SingleMetric() {metric = NULL;};

public:
    /// Return metric synopsis
    virtual std::string toString()
    { return "SingleMetric";  }
    
    static T1* getInstance()
    {
        if(!metric) metric = new T1();
        return metric;
    }
    
    static T1 *metric;
};
    
template <typename T>
T* SingleMetric<T>::metric = NULL;

class BaseMetric: public SingleMetric <BaseMetric>
{
    friend class SingleMetric <BaseMetric>;

protected:
    /// Constructor for Metric class
    BaseMetric() {};
    /// Destructor for Metric class
    virtual ~BaseMetric() {};

public:
    /// Check two messages (completely), return metric
    virtual int compare(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2);

    /// Return metric synopsis
    virtual std::string toString();
        
    virtual std::string operator<< (BaseMetric &_metric);
};

}}}
