/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Pair of Reactions
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/event.hpp>
#include <hw/diagnostics/types.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionPair
{
public:
    ReactionPair();
    ReactionPair(Reaction &_impl_reaction, Reaction &_spec_reaction, Reaction &_collapsed_reaction);

    void set_collapsed(bool collapsed = true);
    bool is_collapsed();

    void set_group(int group);
    int  get_group();

    void add_impl_reaction(Reaction &impl_reaction);
    void add_spec_reaction(Reaction &spec_reaction);
    void add_collapsed_reaction(Reaction &collapsed_reaction);

    Reaction& get_impl_reaction(void);
    Reaction& get_spec_reaction(void);
    Reaction& get_collapsed_reaction(void);

    bool is_spec_reaction_valid();
    bool is_impl_reaction_valid();

    void process(cycle_t reaction_timeout, cycle_t unexpected_reaction_timeout);

protected:
    Reaction _impl_reaction;
    Reaction _spec_reaction;
    Reaction _collapsed_reaction;

    bool _collapsed;
    int  _error;
    int  _group;
};

}}}
