/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Base abstract class for rules
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/diagnostics/reaction_unit_vector.hpp>
#include <hw/diagnostics/metrics/metric.hpp>

namespace cpptesk {
namespace hw {
namespace diagnostics {

const int BASIC_RULE = 0;  // rule1.cpp
    
enum MatchResult
{
    EXACT_MATCH = 0,
    WORST_MATCH = 10000,
    FALSE_MATCH = 10001
};

enum RuleNumber
{
    EMPTY_RULE = -1
};

class Rule
{
public:
    /// Constructor for Rule class
    Rule() {}
    /// Destructor for Rule class
    virtual ~Rule() {}

    /// Update used rule and type in two reactions units
    void assign_rule_number(ReactionUnit &ru1, ReactionUnit &ru2,
                            int rule, CollapseType type);
    /// Check two messages (completely)
    static int compare(const cpptesk::hw::Message &m1, const cpptesk::hw::Message &m2);
    static int compare(const cpptesk::hw::Message *m1, const cpptesk::hw::Message *m2);

    /// Check applicability of the rule
    virtual int check(cpptesk::hw::Reaction &r1, cpptesk::hw::Reaction &r2) = 0;
    /// Apply the rule to the given reaction units
    virtual ReactionUnitVector apply(ReactionUnit &ru1, ReactionUnit &ru2) = 0;
    /// Return rule synopsis
    virtual std::string toString() = 0;
    virtual std::string operator<< (Rule &rule);
};

}}}
