/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Macros for setting interface adapters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/debug.hpp>

////////////////////////////////////////////////////////////////////////////////
// Setting adapters for interface arrays
////////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_SET_IFACE_ARRAY_ADAPTER(IFACE_ADAPTER, iface, full_method_name, array_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < array_size; __cpptesk_i++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER(iface[__cpptesk_i], full_method_name); \
        } \
    } while(false)

#define __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER(IFACE_ADAPTER, iface, full_method_name, i_size, j_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < i_size; __cpptesk_i++) \
        for(int __cpptesk_j = 0; __cpptesk_j < j_size; __cpptesk_j++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER(iface[__cpptesk_i][__cpptesk_j], full_method_name); \
        } \
    } while(false)

////////////////////////////////////////////////////////////////////////////////
// Setting adapters with no listeners for interface arrays
////////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_SET_IFACE_ARRAY_ADAPTER_NO_LISTENER(IFACE_ADAPTER, iface, full_method_name, array_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < array_size; __cpptesk_i++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_NO_LISTENER(iface[__cpptesk_i], full_method_name); \
        } \
    } while(false)

#define __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_NO_LISTENER(IFACE_ADAPTER, iface, full_method_name, i_size, j_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < i_size; __cpptesk_i++) \
        for(int __cpptesk_j = 0; __cpptesk_j < j_size; __cpptesk_j++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_NO_LISTENER(iface[__cpptesk_i][__cpptesk_j], full_method_name); \
        } \
    } while(false)

////////////////////////////////////////////////////////////////////////////////
// Setting adapters for interface arrays of some model
////////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_SET_IFACE_ARRAY_ADAPTER_OF(IFACE_ADAPTER, UnitClass, unit, iface, full_method_name, array_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < array_size; __cpptesk_i++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_OF(UnitClass, unit, iface[__cpptesk_i], full_method_name); \
        } \
    } while(false)

#define __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_OF(IFACE_ADAPTER, UnitClass, unit, iface, full_method_name, i_size, j_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < i_size; __cpptesk_i++) \
        for(int __cpptesk_j = 0; __cpptesk_j < j_size; __cpptesk_j++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_OF(UnitClass, unit, iface[__cpptesk_i][__cpptesk_j], full_method_name); \
        } \
    } while(false)

////////////////////////////////////////////////////////////////////////////////
// Setting adapters with no listeners for interface arrays of some model
////////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_SET_IFACE_ARRAY_ADAPTER_NO_LISTENER_OF(IFACE_ADAPTER, UnitClass, unit, iface, full_method_name, array_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < array_size; __cpptesk_i++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_NO_LISTENER_OF(UnitClass, unit, unit.iface[__cpptesk_i], full_method_name); \
        } \
    } while(false)

#define __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_NO_LISTENER_OF(IFACE_ADAPTER, UnitClass, unit, iface, full_method_name, i_size, j_size) \
    do { \
        for(int __cpptesk_i = 0; __cpptesk_i < i_size; __cpptesk_i++) \
        for(int __cpptesk_j = 0; __cpptesk_j < j_size; __cpptesk_j++) \
        { \
            CPPTESK_SET_##IFACE_ADAPTER##_NO_LISTENER_OF(UnitClass, unit, unit.iface[__cpptesk_i][__cpptesk_j], full_method_name); \
        } \
    } while(false)

////////////////////////////////////////////////////////////////////////////////
// Setting input adapters
////////////////////////////////////////////////////////////////////////////////

/** @def CPPTESK_SET_INPUT_ADAPTER
 * Sets the input adapter (serializer).
 */
#define CPPTESK_SET_INPUT_ADAPTER(iface, full_method_name) \
    registerInputAdapter(iface, (model_t::method_t)&full_method_name, #full_method_name)

/** @def CPPTESK_SET_INPUT_ARRAY_ADAPTER
 * Sets the input adapter (serializer) for the array of interfaces.
 */
#define CPPTESK_SET_INPUT_ARRAY_ADAPTER(iface, full_method_name, array_size) \
    __CPPTESK_SET_IFACE_ARRAY_ADAPTER(INPUT_ADAPTER, iface, full_method_name, array_size)

/** @def CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER
 * Sets the input adapter (serializer) for the 2D array of interfaces.
 */
#define CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER(iface, full_method_name, i_size, j_size) \
    __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER(INPUT_ADAPTER, iface, full_method_name, i_size, j_size)

////////////////////////////////////////////////////////////////////////////////
// Setting output adapters
////////////////////////////////////////////////////////////////////////////////

#if CPPTESKHW_CHECK_VERSION(111010)

  /** @def CPPTESK_SET_OUTPUT_ADAPTER
   * Sets the output adapter (deserializer) with the reaction listener.
   */
  #define CPPTESK_SET_OUTPUT_ADAPTER(iface, full_method_name) \
      do { \
          registerOutputAdapter(iface, (model_t::method_t)&full_method_name, #full_method_name); \
          if(!iface.is_with_listener()) \
          { \
              CPPTESK_CALL_PARALLEL_EX(model_t::listener, iface, CPPTESK_GET_IFACE_MESSAGE(iface), \
                  ::cpptesk::hw::EMPTY_MODEL::START_DONT_UPDATE_IFACE_COVERAGE | ::cpptesk::hw::HIGHEST_PRIORITY); \
              iface.set_with_listener(); \
          } \
      } while(false)

  /** @def CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER
   * Sets the output adapter (deserializer) with no reaction listener.
   */
  #define CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER(iface, full_method_name) \
      registerOutputAdapter(iface, (model_t::method_t)&full_method_name, #full_method_name)

#else // !CPPTESKHW_CHECK_VERSION(111010)

  /** @def CPPTESK_SET_OUTPUT_ADAPTER
   * Registers the output iadapter (deserializer).
   */
  #define CPPTESK_SET_OUTPUT_ADAPTER(iface, full_method_name) \
      registerOutputAdapter(iface, (model_t::method_t)&full_method_name, #full_method_name)

#endif // CPPTESKHW_VERSION_CHECK(111010)

/** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER
 * Sets the output adapter (deserializer) for the array of interfaces.
 */
#define CPPTESK_SET_OUTPUT_ARRAY_ADAPTER(iface, full_method_name, array_size) \
    __CPPTESK_SET_IFACE_ARRAY_ADAPTER(OUTPUT_ADAPTER, iface, full_method_name, array_size)

/** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER
 * Sets the output adapter (deserializer) for the 2D array of interfaces.
 */
#define CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER(iface, full_method_name, i_size, j_size) \
    __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER(OUTPUT_ADAPTER, iface, full_method_name, i_size, j_size)

#if CPPTESKHW_CHECK_VERSION(111010)

  /** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISNENER
   * Sets the output adapter (deserializer) with no reaction listner for the array of interfaces.
   */
  #define CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER(iface, full_method_name, array_size) \
      __CPPTESK_SET_IFACE_ARRAY_ADAPTER_NO_LISTENER(OUTPUT_ADAPTER, iface, full_method_name, array_size)

  /** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISNENER
   * Sets the output adapter (deserializer) with no reaction listner for the 2D array of interfaces.
   */
  #define CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER(iface, full_method_name, i_size, j_size) \
      __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_NO_LISTENER(OUTPUT_ADAPTER, iface, full_method_name, i_size, j_size)

#endif // CPPTESKHW_VERSION_CHECK(111010)

////////////////////////////////////////////////////////////////////////////////
// Setting input adapters for some model
////////////////////////////////////////////////////////////////////////////////

/** @def CPPTESK_SET_INPUT_ADAPTER_OF
 * Sets the input adapter for the given model.
 */
#define CPPTESK_SET_INPUT_ADAPTER_OF(UnitClass, unit, iface, full_method_name) \
    do { \
        unit.registerInputAdapter(unit.iface, reinterpret_cast<UnitClass::model_t&>(*this), \
            (UnitClass::model_t::method_t)&full_method_name, #full_method_name); \
    } while(false)

/** @def CPPTESK_SET_INPUT_ARRAY_ADAPTER_OF
 * Sets the input adapter for the array of interfaces of the given model.
 */
#define CPPTESK_SET_INPUT_ARRAY_ADAPTER_OF(UnitClass, unit, iface, full_method_name, array_size) \
    __CPPTESK_SET_IFACE_ARRAY_ADAPTER_OF(INPUT_ADAPTER, UnitClass, unit, iface, full_method_name, array_size)

/** @def CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER_OF
 * Sets the input adapter for the 2D array of interfaces of the given model.
 */
#define CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER_OF(UnitClass, unit, iface, full_method_name, i_size, j_size) \
    __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_OF(INPUT_ADAPTER, UnitClass, unit, iface, full_method_name, i_size, j_size)

////////////////////////////////////////////////////////////////////////////////
// Setting output adapters for some model
////////////////////////////////////////////////////////////////////////////////

/** @def CPPTESK_SET_OUTPUT_ADAPTER_OF
 * Sets the output adapter for the given model.
 */
#define CPPTESK_SET_OUTPUT_ADAPTER_OF(UnitClass, unit, iface, full_method_name) \
    do { \
        unit.registerOutputAdapter(unit.iface, reinterpret_cast<UnitClass::model_t&>(*this), \
            (UnitClass::model_t::method_t)&full_method_name, #full_method_name); \
        if(!unit.iface.is_with_listener()) \
        { \
            CPPTESK_CALL_PROCESS_OF_EX(reinterpret_cast<UnitClass::model_t&>(unit), \
                UnitClass::listener, unit.iface, unit.CPPTESK_GET_IFACE_MESSAGE(iface), \
                (::cpptesk::hw::EMPTY_MODEL::START_DONT_UPDATE_IFACE_COVERAGE | ::cpptesk::hw::HIGHEST_PRIORITY)); \
            unit.iface.set_with_listener(); \
        } \
    } while(false)

/** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_OF
 * Sets the output adapter for the array of interfaces of the given model.
 */
#define CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_OF(UnitClass, unit, iface, full_method_name, array_size) \
    __CPPTESK_SET_IFACE_ARRAY_ADAPTER_OF(OUTPUT_ADAPTER, UnitClass, unit, iface, full_method_name, array_size)

/** @def CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_OF
 * Sets the output adapter for the 2D array of interfaces of the given model.
 */
#define CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_OF(UnitClass, unit, iface, full_method_name, i_size, j_size) \
    __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_OF(OUTPUT_ADAPTER, UnitClass, unit, iface, full_method_name, i_size, j_size)

#if CPPTESKHW_CHECK_VERSION(111010)

////////////////////////////////////////////////////////////////////////////////
// Setting output adapters (no listeners) for some models
////////////////////////////////////////////////////////////////////////////////

  /** @def CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER_OF
   * Sets the output adapter (no listener) for the given model.
   */
  #define CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER_OF(UnitClass, unit, iface, full_method_name) \
    do { \
        unit.registerOutputAdapter(unit.iface, reinterpret_cast<UnitClass::model_t&>(*this), \
            (UnitClass::model_t::method_t)&full_method_name, #full_method_name); \
    } while(false)

  /** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER_OF
   * Sets the output adapter (no listener) for the array of interfaces of the given model.
   */
  #define CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER_OF(UnitClass, unit, iface, full_method_name, array_size) \
      __CPPTESK_SET_IFACE_ARRAY_ADAPTER_NO_LISTENER_OF(OUTPUT_ADAPTER, UnitClass, unit, iface, full_method_name, array_size)

  /** @def CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER_OF
   * Sets the output adapter (no listener) for the 2D array of interfaces of the given model.
   */
  #define CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER_OF(UnitClass, unit, iface, full_method_name, i_size, j_size) \
      __CPPTESK_SET_IFACE_ARRAY_2D_ADAPTER_NO_LISTENER_OF(OUTPUT_ADAPTER, UnitClass, unit, iface, full_method_name, i_size, j_size)

////////////////////////////////////////////////////////////////////////////////
// Setting default input adapters
////////////////////////////////////////////////////////////////////////////////

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default input adapter.
   */
  #define CPPTESK_SET_DEFAULT_INPUT_ADAPTER(iface) \
      CPPTESK_SET_INPUT_ADAPTER(iface, CPPTESK_DEFAULT_INPUT_ADAPTER(model_t::iface))

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default input adapter for the array of interfaces.
   */
  #define CPPTESK_SET_DEFAULT_INPUT_ARRAY_ADAPTER(iface, array_size) \
      CPPTESK_SET_INPUT_ARRAY_ADAPTER(iface, CPPTESK_DEFAULT_INPUT_ADAPTER(model_t::iface), array_size)

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default input adapter for the 2D array of interfaces.
   */
  #define CPPTESK_SET_DEFAULT_INPUT_ARRAY_2D_ADAPTER(iface, i_size, j_size) \
      CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER(iface, CPPTESK_DEFAULT_INPUT_ADAPTER(model_t::iface), i_size, j_size)

////////////////////////////////////////////////////////////////////////////////
// Setting default output adapters
////////////////////////////////////////////////////////////////////////////////

  /** @def cpptesk_set_default_input_adapter
   * sets the default output adapter.
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ADAPTER(iface) \
      CPPTESK_SET_OUTPUT_ADAPTER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface))

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default output adapter for the array of interfaces.
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_ADAPTER(iface, array_size) \
      CPPTESK_SET_OUTPUT_ARRAY_ADAPTER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface), array_size)

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default output adapter for the 2D array of interfaces.
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER(iface, i_size, j_size) \
      CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface), i_size, j_size)

////////////////////////////////////////////////////////////////////////////////
// Setting default output adapters (no listeners)
////////////////////////////////////////////////////////////////////////////////

  /** @def cpptesk_set_default_input_adapter
   * sets the default output adapter (no listener).
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ADAPTER_NO_LISTENER(iface) \
      CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface))

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default output adapter for the array of interfaces (no listeners).
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_ADAPTER_NO_LISTENER(iface, array_size) \
      CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface), array_size)

  /** @def CPPTESK_SET_DEFAULT_INPUT_ADAPTER
   * Sets the default output adapter for the 2D array of interfaces (no listeners).
   */
  #define CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER(iface, i_size, j_size) \
      CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER(iface, CPPTESK_DEFAULT_OUTPUT_ADAPTER(model_t::iface), i_size, j_size)

#endif // CPPTESK_CHECK_VERSION(111010)

////////////////////////////////////////////////////////////////////////////////
// Short names
////////////////////////////////////////////////////////////////////////////////

#ifdef HW_SHORT_NAMES
    #define SET_INPUT_ADAPTER                                 CPPTESK_SET_INPUT_ADAPTER
    #define SET_INPUT_ARRAY_ADAPTER                           CPPTESK_SET_INPUT_ARRAY_ADAPTER
    #define SET_INPUT_ARRAY_2D_ADAPTER                        CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER
    #define SET_OUTPUT_ADAPTER                                CPPTESK_SET_OUTPUT_ADAPTER
    #define SET_OUTPUT_ARRAY_ADAPTER                          CPPTESK_SET_OUTPUT_ARRAY_ADAPTER
    #define SET_OUTPUT_ARRAY_2D_ADAPTER                       CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER
    #define SET_DEFAULT_INPUT_ADAPTER                         CPPTESK_SET_DEFAULT_INPUT_ADAPTER
    #define SET_DEFAULT_INPUT_ARRAY_ADAPTER                   CPPTESK_SET_DEFAULT_INPUT_ARRAY_ADAPTER
    #define SET_DEFAULT_INPUT_ARRAY_2D_ADAPTER                CPPTESK_SET_DEFAULT_INPUT_ARRAY_2D_ADAPTER
    #define SET_DEFAULT_OUTPUT_ADAPTER                        CPPTESK_SET_DEFAULT_OUTPUT_ADAPTER
    #define SET_DEFAULT_OUTPUT_ARRAY_ADAPTER                  CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_ADAPTER
    #define SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER               CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER
    #define SET_INPUT_ADAPTER_OF                              CPPTESK_SET_INPUT_ADAPTER_OF
    #define SET_INPUT_ARRAY_ADAPTER_OF                        CPPTESK_SET_INPUT_ARRAY_ADAPTER_OF
    #define SET_INPUT_ARRAY_2D_ADAPTER_OF                     CPPTESK_SET_INPUT_ARRAY_2D_ADAPTER_OF
    #define SET_OUTPUT_ADAPTER_OF                             CPPTESK_SET_OUTPUT_ADAPTER_OF
    #define SET_OUTPUT_ARRAY_ADAPTER_OF                       CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_OF
    #define SET_OUTPUT_ARRAY_2D_ADAPTER_OF                    CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_OF
    #define SET_OUTPUT_ADAPTER_NO_LISTENER                    CPPTESK_SET_OUTPUT_ADAPTER_NO_LISTENER
    #define SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER              CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER
    #define SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER           CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER
    #define SET_DEFAULT_OUTPUT_ADAPTER_NO_LISTENER            CPPTESK_SET_DEFAULT_OUTPUT_ADAPTER_NO_LISTENER
    #define SET_DEFAULT_OUTPUT_ARRAY_ADAPTER_NO_LISTENER      CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_ADAPTER_NO_LISTENER
    #define SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER   CPPTESK_SET_DEFAULT_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER
    #define SET_OUTPUT_ADAPTER_NO_LISTENER_OF                 CPPTESK_SET_OUTPUT_ADAPTER_OF_NO_LISTENER
    #define SET_OUTPUT_ARRAY_ADAPTER_NO_LISTENER_OF           CPPTESK_SET_OUTPUT_ARRAY_ADAPTER_OF_NO_LISTENER
    #define SET_OUTPUT_ARRAY_2D_ADAPTER_NO_LISTENER_OF        CPPTESK_SET_OUTPUT_ARRAY_2D_ADAPTER_OF_NO_LISTENER
#endif // HW_SHORT_NAMES

