/*
 * Copyright 2010-2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Pseudo-random number generator
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>
#include <hw/random/generator.hpp>
#include <hw/random/randomizable.hpp>
#include <hw/random/randomization.hpp>
#include <hw/utils/singleton.hpp>

#include <stdint.h>
#include <iostream>

#define __CPPTESK_GET_RANDOM() \
    ::cpptesk::hw::Randomization::get()

#define __CPPTESK_ARRAY(type, ...) ((type[]){ __VA_ARGS__ })
#define __CPPTESK_ARRAY_SIZE(type, ...) (sizeof(__CPPTESK_ARRAY(type, __VA_ARGS__))/sizeof(type))

#define CPPTESK_RANDOM(type) \
    ((type)__CPPTESK_GET_RANDOM().rand<type>())

#define CPPTESK_RANDOM_WIDTH(type, width) \
    ((type)__CPPTESK_GET_RANDOM().rand<type>(width))

#define CPPTESK_RANDOM_FIELD(type, lo, hi) \
    ((type)__CPPTESK_GET_RANDOM().rand<type>((unsigned)(lo), (unsigned)(hi)))

#define CPPTESK_RANDOM_BITS(type, lo, hi) \
    CPPTESK_RANDOM_FIELD(type, lo, hi)

#define CPPTESK_RANDOM_RANGE(type, min, max) \
    ((type)__CPPTESK_GET_RANDOM().rand_range<type>((type)(min), (type)(max)))

#define CPPTESK_RANDOM_ENUM(type, min, max) \
    ((type)CPPTESK_RANDOM_RANGE(int, (int)min, (int)max))

#define CPPTESK_RANDOM_CHOICE(type, ...) \
    ((type)__CPPTESK_GET_RANDOM().rand_choice<type>(__CPPTESK_ARRAY(type, __VA_ARGS__), __CPPTESK_ARRAY_SIZE(type, __VA_ARGS__)))

#define CPPTESK_RANDOM_EXCLUDING(type, ...) \
     ((type)__CPPTESK_GET_RANDOM().rand_excluding<type>(__CPPTESK_ARRAY(type, __VA_ARGS__), __CPPTESK_ARRAY_SIZE(type, __VA_ARGS__)))
   
#define CPPTESK_RANDOM_WIDTH_EXCLUDING(type, width, ...) \
     ((type)__CPPTESK_GET_RANDOM().rand_excluding<type>(width, __CPPTESK_ARRAY(type, __VA_ARGS__), __CPPTESK_ARRAY_SIZE(type, __VA_ARGS__)))
   
#define CPPTESK_RANDOM_FIELD_EXCLUDING(type, lo, hi, ...) \
     ((type)__CPPTESK_GET_RANDOM().rand_excluding<type>(lo, hi, __CPPTESK_ARRAY(type, __VA_ARGS__), __CPPTESK_ARRAY_SIZE(type, __VA_ARGS__)))

#define CPPTESK_RANDOM_BITS_EXCLUDING(type, lo, hi, ...) \
    CPPTESK_RANDOM_FIELD_EXCLUDING(type, lo, hi, __VA_ARGS__)

#define CPPTESK_RANDOM_RANGE_EXCLUDING(type, min, max, ...) \
     ((type)__CPPTESK_GET_RANDOM().rand_range_excluding<type>(min, max, __CPPTESK_ARRAY(type, __VA_ARGS__), __CPPTESK_ARRAY_SIZE(type, __VA_ARGS__)))

#define CPPTESK_RANDOM_ENUM_EXCLUDING(type, min, max, ...) \
    ((type)CPPTESK_RANDOM_RANGE_EXCLUDING(int, (int)min, (int)max, __VA_ARGS__))

#define CPPTESK_RANDOM_INDICATOR(probability) \
    (__CPPTESK_GET_RANDOM().rand_indicator(probability))

#define CPPTESK_WITH_PROBABILITY(probability) \
    if(CPPTESK_RANDOM_INDICATOR(probability))

#define CPPTESK_ELSE_WITH_PROBABILITY(probability) \
    else CPPTESK_WITH_PROBABILITY(probability)

#define CPPTESK_ELSE else

#ifdef HW_SHORT_NAMES
    #define RANDOM                 CPPTESK_RANDOM
    #define RANDOM_WIDTH           CPPTESK_RANDOM_WIDTH
    #define RANDOM_FIELD           CPPTESK_RANDOM_FIELD
    #define RANDOM_BITS            CPPTESK_RANDOM_BITS
    #define RANDOM_RANGE           CPPTESK_RANDOM_RANGE
    #define RANDOM_ENUM            CPPTESK_RANDOM_ENUM
    #define RANDOM_CHOICE          CPPTESK_RANDOM_CHOICE
    #define RANDOM_EXCLUDING       CPPTESK_RANDOM_EXCLUDING
    #define RANDOM_WIDTH_EXCLUDING CPPTESK_RANDOM_WIDTH_EXCLUDING
    #define RANDOM_FIELD_EXCLUDING CPPTESK_RANDOM_FIELD_EXCLUDING
    #define RANDOM_BITS_EXCLUDING  CPPTESK_RANDOM_BITS_EXCLUDING
    #define RANDOM_RANGE_EXCLUDING CPPTESK_RANDOM_RANGE_EXCLUDING
    #define RANDOM_ENUM_EXCLUDING  CPPTESK_RANDOM_ENUM_EXCLUDING
    #define RANDOM_INDICATOR       CPPTESK_RANDOM_INDICATOR
    #define WITH_PROBABILITY       CPPTESK_WITH_PROBABILITY
    #define ELSE_WITH_PROBABILITY  CPPTESK_ELSE_WITH_PROBABILITY
    #define ELSE                   CPPTESK_ELSE
#endif // HW_SHORT_NAMES

namespace cpptesk {
namespace hw {

}} // namespace cpptesk::hw

