/*
 * Copyright 2013 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Random number generator
///////////////////////////////////////////////////////////////////////////////

#pragma once

// C header from the netfsm project.
extern "C"
{
  #include <rand/random.h>
}

#include <hw/defs.hpp>
#include <hw/utils/singleton.hpp>

namespace cpptesk {
namespace hw {

/**
 * Modified lagged Fibonacci generator (MLFG). 
 */
class Generator: public Singleton<Generator>
{
    friend class Singleton<Generator>;

    /// Constructor of the MLFG generator.
    Generator()
    {
        // Initialize the generator state.
        init_MLFG(&_generator);
        // Set the default seed.
        seed_MLFG(&_generator, modify_seed(0));
    }

public:
    /// Returns the next random number.
    uint32_t next() { return next_MLFG(&_generator); }
    /// Sets the seed of the random number generator.
    void seed(uint32_t seed) { seed_MLFG(&_generator, modify_seed(seed)); }

private:
    /// Modifies the seed (generators of the same type are used in the netfsm engines;
    /// to avoid the correlation, the seed is modified).
    uint32_t modify_seed(uint32_t seed) const  { return 1664525 * seed + 1013904223; }

private:
    // The generator itself (borrowed from the netfsm project).
    MLFG _generator;
};

}} // namespace cpptesk::hw

